using System;

namespace GeneticAlgorithms
{
	/// <summary>
	/// Counter is just a high-resolution stopwatch for timing operations.
	/// Slightly modified code created by the legendary Eric Gunnerson of Microsoft.
	/// </summary>
	public class Counter 
	{
		long elapsedCount = 0;
		long startCount = 0;
		long lastLapCount = 0;

		public void Start()
		{
			startCount = 0;
			QueryPerformanceCounter(ref startCount);
		}
		
		public void Stop()
		{
			long stopCount = 0;
			QueryPerformanceCounter(ref stopCount);

			elapsedCount += (stopCount - startCount);
		}

		public void Clear()
		{
			elapsedCount = 0;
		}

		public double Seconds
		{
			get
			{
				long freq = 0;
				QueryPerformanceFrequency(ref freq);
				return((double) elapsedCount / (double) freq);
			}
		}

		public override string ToString()
		{
			return String.Format("{0} seconds", Seconds);
		}
		
		public double Lap
		{
			get
			{
				long freq=0;
				long elapsed=lastLapCount;
				QueryPerformanceFrequency(ref freq);
				QueryPerformanceCounter(ref lastLapCount);
				return ((double)(lastLapCount-elapsed)/(double)freq);
			}
		}

		public static long Frequency 
		{
			get 
			{
				long freq = 0;
				QueryPerformanceFrequency(ref freq);
				return freq;
			}
		}
		public static long Value 
		{
			get 
			{
				long count = 0;
				QueryPerformanceCounter(ref count);
				return count;
			}
		}

		[System.Runtime.InteropServices.DllImport("KERNEL32")]
		private static extern bool QueryPerformanceCounter(  ref long lpPerformanceCount);

		[System.Runtime.InteropServices.DllImport("KERNEL32")]
		private static extern bool QueryPerformanceFrequency( ref long lpFrequency);                     
	}
}
